""" Test class for wizard """
import logging

from django.test import TestCase
# from django.contrib.auth.models import AnonymousUser

from articles.models import User, UserActivityLogs
from articles.test_with_articles import CreateArticlesWithTagsTestCase
from articles.wizard.manipulators import create_users_in_db, deactivate_user

logger = logging.getLogger(__name__)


class OverviewView(CreateArticlesWithTagsTestCase):
    """ Test the overview view """

    def test_plain_context_data(self):
        """ test context data with DB Data """
        logger.info("started OverviewView.test_context_data")
        response = self.client.get('/wizard/overview')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['articles']['count'], 1)
        self.assertEqual(response.context['articles']['wrongly_parsed_count'], 0)
        self.assertEqual(response.context['media']['count'], 1)
        self.assertEqual(response.context['media']['top'][0].name, 'maz')
        self.assertEqual(response.context['tags']['count'], 2)
        self.assertEqual(response.context['tags']['top'][0].name, 'Test1')
        self.assertEqual(response.context['files']['count'], 1)
        self.assertEqual(response.context['users']['count'], 1)
        self.assertEqual(response.context['ratio']['apt'], 0.5)
        self.assertEqual(response.context['ratio']['tpa'], 2)


class HomeView(TestCase):
    """ Test the home view """

    def test_plain_context_data(self):
        """ test context data """
        logger.info("started HomeView.test_plain_context_data")
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)
        self.assertTrue(len(response.context['site_urls']['articles']) > 0)
        self.assertTrue(len(response.context['site_urls']['pr_upload']) > 0)
        self.assertTrue(len(response.context['site_urls']['wizard']) > 0)
        self.assertTrue(len(response.context['site_urls']['projekt']) > 0)


class Manipulator(TestCase):
    """ Tests methods in manipulators """

    def test_create_user(self):
        """ tests user creation """
        logger.info("started Manipulator.test_create_user")
        create_users_in_db(1)
        self.assertEqual(User.objects.all().count(), 2)
        create_users_in_db(3)
        self.assertEqual(User.objects.all().count(), 5)

    def test_deactivate_one_user(self):
        """ tests how one user can be deactivated"""
        logger.info("started Manipulator.test_deactivate_one_user")
        ids = create_users_in_db(1)
        user = User.objects.get(id_tag=ids[0])
        deactivate_user(user.id)
        self.assertEqual(User.objects.filter(activated=False).count(), 1)
        self.assertEqual(User.objects.filter(activated=True).count(), 1)

    def test_deactivate_multiple_user(self):
        """ tests how users can be deactivated"""
        logger.info("started Manipulator.test_deactivate_multiple_user")
        create_users_in_db(3)
        user_ids = []
        for user in User.objects.all():
            user_ids.append(user.id)
        deactivate_user(*user_ids)
        self.assertEqual(User.objects.filter(activated=False).count(), 4)
        self.assertEqual(User.objects.filter(activated=True).count(), 0)

    def test_create_user_api(self):
        """tests user creation via api"""
        logger.info("started Manipulator.test_create_user_api")
        response = self.client.post('/articles/user/create', {})
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(
            str(response.content, encoding='utf8'),
            {'user-id': '1'}
        )
        new_user_ids = create_users_in_db(1)
        response = self.client.post('/articles/user/create', {})
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(
            str(response.content, encoding='utf8'),
            {'user-id': str(new_user_ids[0])}
        )


class SimulatorView(TestCase):
    """ Testclass for the SimulatorView """

    def test_plain_view(self):
        """ tests the plain view """
        logger.info("started SimulatorView.test_plain_view")
        response = self.client.get('/wizard/simulator')
        self.assertEqual(response.status_code, 200)

    # def test_user_details(self):
    #     """ tests if you can have a detailed view """
    #     logger.info("started SimulatorView.test_user_details")
    #     create_users_in_db(1)
    #     user = User.objects.get(id=1)
    #     post_data = {'all_users': [user.id_tag]}
    #     response = self.client.post('/wizard/simulator', post_data)
    #     self.assertTrue(isinstance(response.context["user"], AnonymousUser))
    #     self.assertEqual(response.status_code, 200)


class NightlyPersistLogsView(TestCase):
    """ Testclass for NightlyPersistLogs """

    def test_api(self):
        """ tests the plain view """
        logger.info("started NightlyPersistLogsView.test_api")
        # check if log database is empty
        user_activity_count = UserActivityLogs.objects.all().count()
        self.assertEqual(0, user_activity_count)
        # add a exemplary log to the log file
        logger.info('"GET /articles/top/media?mediaOverviewType=usertags&user=99999 HTTP/1.1" 200 304')
        # trigger the migration from log file to log database
        response = self.client.post('/nightly/persist_logs', {})
        self.assertEqual(response.status_code, 200)
        # check if there are logs in the log database
        user_activity = UserActivityLogs.objects.filter(user_id='99999').order_by('-timestamp')
        self.assertTrue(user_activity.count() > 0)
        self.assertEqual(user_activity[0].action, 'GET /articles/top/media?mediaOverviewType=usertags&user=99999 HTTP/1.1')
